import ast
import ConfigParser
import os
from types import MethodType


def get_prepared_config(
        profile,
        region,
        ssl_verification,
        adfs_host,
        rsa_keys,
        output_format,
):
    """
    Prepares ADF configuration for login task.
    The task comprises steps as follows:
        - default configuration preparation,
        - creating aws cli configuration files, if needed
        - loading adf configuration for specified aws profiles
    The configuration is stored in ctx.adfs_config attribute
    :param output_format: output format used by aws cli
    :param rsa_keys: rsa keys used to decrypt / entrypt user's credentials
    :param adfs_host: fqdn of adfs host that will be used to authenticate user
    :param ssl_verification: SSL certificate verification: Whether or not strict certificate
                             verification is done, False should only be used for dev/test
    :param region: The default AWS region that this script will connect
                   to for all API calls
    :param profile: aws cli profile
    """
    adfs_config.profile = profile
    adfs_config.ssl_verification = ssl_verification
    adfs_config.region = region
    adfs_config.adfs_host = adfs_host
    adfs_config.id_rsa_location = rsa_keys[0]
    adfs_config.id_rsa_pub_location = rsa_keys[1]
    adfs_config.output_format = output_format
    _create_base_aws_cli_config_files_if_needed(adfs_config)
    _load_adfs_config_from_stored_profile(adfs_config, profile)

    return adfs_config


def _create_adfs_default_config():
    config = type('', (), {})()
    # region: The default AWS region that this script will connect
    # to for all API calls
    config.region = 'eu-central-1'

    # aws cli profile
    config.profile = 'adfs'

    # output format: The AWS CLI output format that will be configured in the
    # adf profile (affects subsequent CLI calls)
    config.output_format = 'json'

    # aws credential location: The file where this script will store the temp
    # credentials under the configured profile
    config.aws_config_root = os.path.join(os.path.expanduser('~'), '.aws')
    config.aws_credentials_location = os.path.join(config.aws_config_root, 'credentials')
    config.aws_config_location = os.path.join(config.aws_config_root, 'config')

    # rsa identity file location used to decrypt / encrypt password for storage purposes
    config.id_rsa_root = os.path.join(os.path.expanduser('~'), '.ssh')
    config.id_rsa_location = os.path.join(config.id_rsa_root, 'id_rsa')
    config.id_rsa_pub_location = os.path.join(config.id_rsa_root, 'id_rsa.pub')

    # SSL certificate verification: Whether or not strict certificate
    # verification is done, False should only be used for dev/test
    config.ssl_verification = True

    # AWS role arn
    config.role_arn = None

    config.adfs_host = None

    config.adfs_user = None
    config.adfs_password = None
    config.adfs_credentials_loaded = False

    return config


def _load_adfs_config_from_stored_profile(adfs_config, profile):
    def load_from_config(config_location, profile, loader):
        config = ConfigParser.RawConfigParser()
        config.read(config_location)
        if config.has_section(profile):
            def get_or(self, profile, option, default_value):
                if self.has_option(profile, option):
                    return self.get(profile, option)
                return default_value

            setattr(config, get_or.__name__, MethodType(get_or, config, type(config)))
            loader(config, profile)

        del config

    def load_credentials(config, profile):
        adfs_config.adfs_user = config.get_or(profile, 'adfs_config.adfs_user', None)
        adfs_config.adfs_password = config.get_or(profile, 'adfs_config.adfs_password', None)

        adfs_config.adfs_credentials_loaded = adfs_config.adfs_user and adfs_config.adfs_password

    def load_config(config, profile):
        adfs_config.region = config.get_or(profile, 'region', adfs_config.region)
        adfs_config.output_format = config.get_or(profile, 'output', adfs_config.output_format)
        adfs_config.ssl_verification = ast.literal_eval(config.get_or(
            profile, 'adfs_config.ssl_verification',
            adfs_config.ssl_verification))
        adfs_config.role_arn = config.get_or(profile, 'adfs_config.role_arn', adfs_config.role_arn)
        adfs_config.adfs_host = config.get_or(profile, 'adfs_config.adfs_host', adfs_config.adfs_host)

    load_from_config(adfs_config.aws_credentials_location, profile, load_credentials)
    load_from_config(adfs_config.aws_config_location, 'profile ' + profile, load_config)


def _create_base_aws_cli_config_files_if_needed(adfs_config):
    def touch(fname, mode=0o600):
        flags = os.O_CREAT | os.O_APPEND
        with os.fdopen(os.open(fname, flags, mode)) as f:
            try:
                os.utime(fname, None)
            finally:
                f.close()

    if not os.path.exists(adfs_config.aws_config_root):
        os.mkdir(adfs_config.aws_config_root, 0o700)

    if not os.path.exists(adfs_config.aws_credentials_location):
        touch(adfs_config.aws_credentials_location)

    if not os.path.exists(adfs_config.aws_config_location):
        touch(adfs_config.aws_config_location)


adfs_config = _create_adfs_default_config()



import base64

from Crypto.PublicKey import RSA
from Crypto.Random.random import StrongRandom


def load_key(key_file):
    try:
        content = key_file.read()
    finally:
        key_file.close()

    return RSA.importKey(content)


def encrypt(text, pub_key):
    encrypted_text, = pub_key.encrypt(text, StrongRandom().randint(0, 100000))
    return base64.b64encode(encrypted_text)


def decrypt(encrypted, priv_key):
    return priv_key.decrypt(base64.b64decode(encrypted))



import ConfigParser

import click

from .prepare import adfs_config


@click.command(name='list')
def list_profiles():
    """
    lists available profiles
    """

    config = ConfigParser.RawConfigParser()
    config.read(adfs_config.aws_credentials_location)

    profiles = config.sections()
    if len(profiles) < 1:
        click.echo('No defined profiles')
    else:
        click.echo('Available profiles:')
        for profile in profiles:
            click.echo('    * {}'.format(profile))



import ConfigParser
import base64
import getpass
import itertools
import xml.etree.ElementTree as ET

import boto3
import botocore
import bs4
import click
import requests
import requests_ntlm
from botocore import client

from . import crypt
from . import prepare
from .prepare import adfs_config

# The initial URL that starts the authentication process.
_IDP_ENTRY_URL = 'https://{}/adfs/ls/IdpInitiatedSignOn.aspx?loginToRp=urn:amazon:webservices'


@click.command()
@click.option(
    '--profile',
    default=lambda: adfs_config.profile,
    help='AWS cli profile that will be authenticated.\n'
         'After successful authentication just use:\n'
         'aws --profile <authenticated profile> <service> ...',
)
@click.option(
    '--region',
    default=lambda: adfs_config.region,
    help='The default AWS region that this script will connect\n'
         'to for all API calls',
)
@click.option(
    '--ssl-verification/--no-ssl-verification',
    default=lambda: adfs_config.ssl_verification,
    help='SSL certificate verification: Whether or not strict certificate\n'
         'verification is done, False should only be used for dev/test',
)
@click.option(
    '--adfs-host',
    default=lambda: adfs_config.adfs_host,
    help='For the first time for a profile it has to be provided, next time for the same profile\n'
         'it will be loaded from the stored configuration',
)
@click.option(
    '--rsa-keys',
    default=lambda: (adfs_config.id_rsa_location, adfs_config.id_rsa_pub_location),
    type=(file, file),
    help='Private and public key locations used to decrypt and encrypt credentials into storage'
)
@click.option(
    '--output-format',
    default=lambda: adfs_config.output_format,
    type=click.Choice(['json', 'text', 'table']),
    help='Output format used by aws cli',
)
def login(
        profile,
        region,
        ssl_verification,
        adfs_host,
        rsa_keys,
        output_format,
):
    """
    Authenticates an user with active directory credentials
    """
    config = prepare.get_prepared_config(profile, region, ssl_verification, adfs_host, rsa_keys, output_format)

    _verification_checks(config)

    username, password = _get_user_credentials(config)
    principal_roles, assertion = _authenticate(config, username, password)

    pub_key = crypt.load_key(config.id_rsa_pub_location)
    config.adfs_user = crypt.encrypt(text=username, pub_key=pub_key)
    config.adfs_password = crypt.encrypt(text=password, pub_key=pub_key)

    username = '########################################'
    del username
    password = '########################################'
    del password

    principal_arn, config.role_arn = _chosen_role_to_assume(config, principal_roles)

    # Use the assertion to get an AWS STS token using Assume Role with SAML
    conn = boto3.client('sts', config=client.Config(signature_version=botocore.UNSIGNED))
    aws_session_token = conn.assume_role_with_saml(
        RoleArn=config.role_arn,
        PrincipalArn=principal_arn,
        SAMLAssertion=assertion,
        DurationSeconds=3600,
    )

    _store(config, aws_session_token)
    _emit_summary(config)


def _emit_summary(config):
    click.echo(
        """
        Prepared ADFS configuration as follows:
            * AWS Cli profile                           : '{}'
            * AWS region                                : '{}'
            * output format                             : '{}'
            * ssl verification during authentication was: '{}'
            * selected role_arn                         : '{}'
            * ADFS host used for authentication         : '{}'
        """.format(
            config.profile,
            config.region,
            config.output_format,
            'ENABLED' if config.ssl_verification else 'DISABLED',
            config.role_arn,
            config.adfs_host,
        )
    )


def _get_user_credentials(config):
    if config.adfs_credentials_loaded:
        priv_key = crypt.load_key(config.id_rsa_location)
        username = crypt.decrypt(config.adfs_user, priv_key)
        password = crypt.decrypt(config.adfs_password, priv_key)

        del priv_key
    else:
        username = raw_input('Username: ')
        password = getpass.getpass()

    return username, password


def _authenticate(config, username, password):
    # Initiate session handler
    session = requests.Session()

    # Programmatically get the SAML assertion
    # Set up the NTLM authentication handler by using the provided credential
    session.auth = requests_ntlm.HttpNtlmAuth(username, password)

    # Opens the initial AD FS URL and follows all of the HTTP302 redirects
    response = session.post(
        _IDP_ENTRY_URL.format(config.adfs_host),
        verify=config.ssl_verification,
        data={
            'UserName': username,
            'Password': password,
            'AuthMethod': 'urn:amazon:webservices'
        }
    )

    del username
    password = '###################################################'
    del password

    # Decode the response and extract the SAML assertion
    soup = bs4.BeautifulSoup(response.text.decode('utf8'), 'html.parser')
    assertion = None

    # Look for the SAMLResponse attribute of the input tag (determined by
    # analyzing the debug print lines above)
    for inputtag in soup.find_all('input'):
        if inputtag.get('name') == 'SAMLResponse':
            assertion = inputtag.get('value')

    if not assertion:
        click.echo('Wrong authentication. Username or password doesn\'t match')
        exit(-1)

    # Parse the returned assertion and extract the authorized roles
    root = ET.fromstring(base64.b64decode(assertion))

    aws_roles = map(
        lambda saml2attributevalue: saml2attributevalue.text,
        itertools.chain.from_iterable(
            map(
                lambda saml2attribute: list(
                    saml2attribute.iter('{urn:oasis:names:tc:SAML:2.0:assertion}AttributeValue')),
                filter(
                    lambda saml2attribute: saml2attribute.get('Name') == 'https://aws.amazon.com/SAML/Attributes/Role',
                    root.iter('{urn:oasis:names:tc:SAML:2.0:assertion}Attribute')
                ),
            )
        )
    )

    # Note the format of the attribute value is principal_arn, role_arn
    principal_roles = map(
        lambda chunks: (chunks[0], chunks[1]),
        filter(
            lambda chunks: 'saml-provider' in chunks[0],
            map(
                lambda role: role.split(','),
                aws_roles,
            )
        )
    )

    return principal_roles, assertion


def _store(config, aws_session_token):
    def store_config(profile, config_location, storer):
        config_file = ConfigParser.RawConfigParser()
        config_file.read(config_location)

        if not config_file.has_section(profile):
            config_file.add_section(profile)

        storer(config_file, profile)

        with open(config_location, 'w+') as f:
            try:
                config_file.write(f)
            finally:
                f.close()

    def credentials_storer(config_file, profile):
        config_file.set(profile, 'aws_access_key_id', aws_session_token['Credentials']['AccessKeyId'])
        config_file.set(profile, 'aws_secret_access_key', aws_session_token['Credentials']['SecretAccessKey'])
        config_file.set(profile, 'aws_session_token', aws_session_token['Credentials']['SessionToken'])
        config_file.set(profile, 'adfs_config.adfs_user', config.adfs_user)
        config_file.set(profile, 'adfs_config.adfs_password', config.adfs_password)

    def config_storer(config_file, profile):
        config_file.set(profile, 'region', config.region)
        config_file.set(profile, 'output', config.output_format)
        config_file.set(profile, 'adfs_config.ssl_verification', config.ssl_verification)
        config_file.set(profile, 'adfs_config.adfs_host', config.adfs_host)
        config_file.set(profile, 'adfs_config.role_arn', config.role_arn)
        config_file.set(profile, 'source_profile', config.profile)

    store_config(config.profile, config.aws_credentials_location, credentials_storer)
    store_config('profile {}'.format(config.profile), config.aws_config_location, config_storer)


def _verification_checks(config):
    if not config.adfs_host:
        click.echo('\'--adfs-host\' parameter need to be supplied')
        exit(-1)


def _chosen_role_to_assume(config, principal_roles):
    chosen_principal_role = filter(
        lambda (_, role_arn): config.role_arn == role_arn,
        principal_roles
    )

    if chosen_principal_role:
        chosen_role_arn = chosen_principal_role[0][0]
        chosen_principal_arn = chosen_principal_role[0][1]
        return chosen_role_arn, chosen_principal_arn

    if len(principal_roles) > 1:
        click.echo('Please choose the role you would like to assume:')
        i = 0
        for (principal_arn, role_arn) in principal_roles:
            role_name = role_arn.split(':role/')[1]
            click.echo('    [ {} -> {} ]: {}'.format(role_name.ljust(30, ' ' if i % 2 == 0 else '.'), i, role_arn))
            i += 1

        selected_index = int(raw_input('Selection: '))

        chosen_principal_arn = principal_roles[selected_index][0]
        chosen_role_arn = principal_roles[selected_index][1]
    else:
        chosen_principal_arn = principal_roles[0][0]
        chosen_role_arn = principal_roles[0][1]

    return chosen_principal_arn, chosen_role_arn



import click

from . import list_profiles
from . import login
from . import reset


@click.group()
def cli():
    pass


cli.add_command(list_profiles.list_profiles)
cli.add_command(login.login)
cli.add_command(reset.reset)



#!/usr/bin/env python

import codecs
from os import path

from setuptools import setup

version = '0.0.3'

setup(
    name='aws-adfs',
    version=version,
    description='AWS Cli authenticator via ADFS - small command-line tool '
                'to authenticate via ADFS and assume chosen role',
    long_description=codecs.open(
        path.join(path.abspath(path.dirname(__file__)), 'README.md'),
        mode='r',
        encoding='utf-8'
    ).read(),
    url='https://github.com/venth/aws-adfs',
    download_url='https://github.com/venth/aws-adfs/tarball/{}'.format(version),
    author='Venth',
    author_email='artur.krysiak.warszawa@gmail.com',
    maintainer='Venth',
    keywords='aws adfs console tool',
    packages=['aws_adfs'],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Python Software Foundation License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
    ],

    install_requires=[
        'click==6.6',
        'beautifulsoup4==4.4.1',
        'boto3==1.3.1',
        'requests_ntlm==0.3.0',
        'requests==2.10.0',
        'pycrypto==2.6.1',
        'Crypto==1.4.1',
    ],
    entry_points={
        'console_scripts': ['aws-adfs=aws_adfs.commands:cli']
    },
    include_package_data=True,
)



import ConfigParser

import click

from .prepare import adfs_config


@click.command()
@click.option(
    '--profile',
    default=lambda: adfs_config.profile,
    help='AWS cli profile that will be removed'
)
def reset(profile):
    """
    removes stored profile
    """
    _clear_credentials(adfs_config, profile)
    click.echo('Profile: \'{}\' has been wiped out'.format(profile))


def _clear_credentials(config, profile):
    def store_config(config_location, storer):
        config_file = ConfigParser.RawConfigParser()
        config_file.read(config_location)

        if not config_file.has_section(profile):
            config_file.add_section(profile)

        storer(config_file)

        with open(config_location, 'w+') as f:
            try:
                config_file.write(f)
            finally:
                f.close()

    def profile_remover(config_file):
        config_file.remove_section(profile)
        config_file.remove_section('profile {}'.format(profile))

    store_config(config.aws_credentials_location, profile_remover)
    store_config(config.aws_config_location, profile_remover)
